%INERTIA_TERM
% Contributions of inertia terms 
% Output: cu0, cv0

% Indexing convention in staggered grid:
%  +--k+1--+ 
%  |       |
%  j  jk  j+1
%  |       |
%  +---k---+ 

if n < 2, tic, end

cu0 = zeros(size(XU)); cv0 = zeros(size(XV));

up = reshape(u0,size(XU')); up = up';		% Two-index reordering of u0
vp = reshape(v0,size(XV')); vp = vp';		%  and v0

%up = zeros(size(XU)); vp = YV;

% Velocity field in cell vertices:
ucv = [up;up(K,:)];			% ucv: convecting u in cell vertices
uav = [up;up(K,:)];			% uav: average u in cell vertices	
ucv(2:K,:) = (ucv(1:K-1,:).*DYU(1:K-1,:) + ucv(2:K,:).*DYU(2:K,:))...
  ./(DYU(1:K-1,:) + DYU(2:K,:));
uav(2:K,:) = (uav(1:K-1,:) + uav(2:K,:))/2;
vcv = [vp,vp(:,J)];			% vcv: convecting v in cell vertices  
vav = [vp,vp(:,J)];			% vav: average v in cell vertices  
vcv(:,2:J) = (vcv(:,1:J-1).*DXV(:,1:J-1) + vcv(:,2:J).*DXV(:,2:J))...
  ./(DXV(:,1:J-1) + DXV(:,2:J));
vav(:,2:J) = (vav(:,1:J-1) + vav(:,2:J))/2;
uav(1,:) = 0; uav(K+1,:) = 0; uav(:,1) = 0; uav(:,J+1) = 0;
ucv(1,:) = 0; ucv(K+1,:) = 0; ucv(:,1) = 0; ucv(:,J+1) = 0;
vav(1,:) = 0; vav(K+1,:) = 0; vav(:,1) = 0; vav(:,J+1) = 0;
vcv(1,:) = 0; vcv(K+1,:) = 0; vcv(:,1) = 0; vcv(:,J+1) = 0;

% Application of boundary corrections:
for seg = 1:length(ybc(1,:))
  if ybc(1,seg) == 2				% Inflow at left boundary
    k = 1+kseg(seg);      
    if k == 1 
      ucv(k,1) = ucv(k,1) + uleft(k)/2; vcv(k,1) = vcv(k,1) + vleft(k)/2; 
    else
      ucv(k,1) = ucv(k,1) + uleft(k)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1));
      vcv(k,1) = vcv(k,1) + vleft(k)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
    end
    uav(k,1) = uav(k,1) + uleft(k)/2; vav(k,1) = vav(k,1) + vleft(k)/2;
    k = 2+kseg(seg):kseg(seg+1);  ucv(k,1) = (uleft(k) + uleft(k-1))/2;
    uav(k,1) = ucv(k,1);
    vcv(k,1) = vleft(k); vav(k,1) = vcv(k,1);
    k = kseg(seg+1)+1;   
    if k == K+1
      ucv(k,1) = ucv(k,1) + uleft(k-1)/2; vcv(k,1) = vcv(k,1) + vleft(k)/2;
    else
      ucv(k,1) = ucv(k,1) + uleft(k-1)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
      vcv(k,1) = vcv(k,1) + vleft(k)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
    end
    uav(k,1) = uav(k,1) + uleft(k-1)/2; vav(k,1) = vav(k,1) + vleft(k)/2;
  elseif ybc(1,seg) == 3			% Outflow at left boundary
    k = 1+kseg(seg);  
    if k == 1
      ucv(k,1) = ucv(k,1) + up(k,1)/2; vcv(k,1) = vcv(k,1) + vp(k,1)/2; 
    else
      ucv(k,1) = ucv(k,1) + up(k,1)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
      vcv(k,1) = vcv(k,1) + vp(k,1)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
    end
    uav(k,1) = uav(k,1) + up(k,1)/2; vav(k,1) = vav(k,1) + vp(k,1)/2; 
    k = 2+kseg(seg):kseg(seg+1);  
    ucv(k,1) = (up(k,1) + up(k-1,1))/2;    vcv(k,1) = vp(k,1);
    uav(k,1) = ucv(k,1);   		   vav(k,1) = vp(k,1);
    k = kseg(seg+1)+1;   
    if k == K+1
      ucv(k,1) = ucv(k,1) + up(k-1,1)/2; vcv(k,1) = vcv(k,1) + vp(k,1)/2; 
    else
      ucv(k,1) = ucv(k,1) + up(k-1,1)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
      vcv(k,1) = vcv(k,1) + vp(k,1)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
    end
    uav(k,1) = uav(k,1) + up(k-1,1)/2; vav(k,1) = vav(k,1) + vp(k,1)/2; 
  elseif ybc(1,seg) == 1			% No-slip at left boundary
    % Do nothing
  else
    error('Wrong ybc')
  end
  if ybc(2,seg) == 2  			% Inflow at right boundary
    k = 1+kseg(seg); 
    if k == 1
      ucv(k,J+1) = ucv(k,J+1) + uright(k)/2; 
      vcv(k,J+1) = vcv(k,J+1) + vright(k)/2; 
    else
      ucv(k,J+1) = ucv(k,J+1) + uright(k)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
      vcv(k,J+1) = vcv(k,J+1) + vright(k)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
    end
    uav(k,J+1) = uav(k,J+1) + uright(k)/2; 
    vav(k,J+1) = vav(k,J+1) + vright(k)/2; 
    k = 2+kseg(seg):kseg(seg+1);   
    ucv(k,J+1) = (uright(k) + uright(k-1))/2;  vcv(k,J+1) = vright(k);
    uav(k,J+1) = ucv(k,J+1);		       vav(k,J+1) = vright(k);
    k = kseg(seg+1)+1;   
    if k == K+1
      ucv(k,J+1) = ucv(k,J+1) + uright(k-1)/2; 
      vcv(k,J+1) = vcv(k,J+1) + vright(k)/2;
    else
      ucv(k,J+1) = ucv(k,J+1) + uright(k-1)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
      vcv(k,J+1) = vcv(k,J+1) + vright(k)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
    end  
    uav(k,J+1) = uav(k,J+1) + uright(k-1)/2; 
    vav(k,J+1) = vav(k,J+1) + vright(k)/2;
  elseif ybc(2,seg) == 3	 	  	% Outflow at right boundary
    k = 1+kseg(seg); 
    if k == 1,      ucv(k,J+1) = ucv(k,J+1) + up(k,J+1)/2;
      vcv(k,J+1) = vcv(k,J+1) + vp(k,J)/2; 
    else
      ucv(k,J+1) = ucv(k,J+1) + up(k,J+1)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
      vcv(k,J+1) = vcv(k,J+1) + vp(k,J)*DYU(k,1)/(DYU(k,1) + DYU(k-1,1)); 
    end
    uav(k,J+1) = uav(k,J+1) + up(k,J+1)/2; vav(k,J+1) = vav(k,J+1) + vp(k,J)/2; 
    k = 2+kseg(seg):kseg(seg+1);   
    ucv(k,J+1) = (up(k,J+1) + up(k-1,J+1))/2;  vcv(k,J+1) = vp(k,J);
    uav(k,J+1) = ucv(k,J+1);  vav(k,J+1) = vp(k,J);
    k = kseg(seg+1)+1; 
    if k == K+1,        ucv(k,J+1) = ucv(k,J+1) + up(k-1,J+1)/2; 
      vcv(k,J+1) = vcv(k,J+1) + vp(k,J)/2;
    else
      ucv(k,J+1) = ucv(k,J+1) + up(k-1,J+1)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
      vcv(k,J+1) = vcv(k,J+1) + vp(k,J)*DYU(k-1,1)/(DYU(k,1) + DYU(k-1,1));
    end
    uav(k,J+1) = uav(k,J+1) + up(k-1,J+1)/2; 
    vav(k,J+1) = vav(k,J+1) + vp(k,J)/2;
  elseif ybc(2,seg) == 1			% No-slip at right boundary 
    % Do nothing
  else
    error('Wrong ybc')
  end
end
for seg = 1:length(xbc(1,:))
  if xbc(1,seg) == 2  			% Inflow at lower boundary
    j = 1+jseg(seg);  
    if j == 1
      vcv(1,j) = vcv(1,j) + vlower(j)/2; ucv(1,j) = ucv(1,j) + ulower(j)/2; 
    else
      ucv(1,j) = ucv(1,j) + ulower(j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(1,j) = vcv(1,j) + vlower(j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
    end
    vav(1,j) = vav(1,j) + vlower(j)/2; uav(1,j) = uav(1,j) + ulower(j)/2; 
    j = 2+jseg(seg):jseg(seg+1);
    ucv(1,j) = ulower(j); vcv(1,j) = (vlower(j) + vlower(j-1))/2; 
    uav(1,j) = ulower(j); vav(1,j) = (vlower(j) + vlower(j-1))/2; 
    j = jseg(seg+1)+1;
    if j == J+1
      vcv(1,j) = vcv(1,j) + vlower(j-1)/2; ucv(1,j) = ucv(1,j) + ulower(j)/2;
    else 
      ucv(1,j) = ucv(1,j) + ulower(j)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(1,j) = vcv(1,j) + vlower(j-1)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1));
    end 
    vav(1,j) = vav(1,j) + vlower(j-1)/2; uav(1,j) = uav(1,j) + ulower(j)/2;
  elseif xbc(1,seg) == 3	 	  	% Outflow at lower boundary
    j = 1+jseg(seg);
    if j == 1
      vcv(1,j) = vcv(1,j) + vp(1,j)/2; ucv(1,j) = ucv(1,j) + up(1,j)/2; 
    else
      ucv(1,j) = ucv(1,j) + up(1,j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(1,j) = vcv(1,j) + vp(1,j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
    end
    vav(1,j) = vav(1,j) + vp(1,j)/2; uav(1,j) = uav(1,j) + up(1,j)/2; 
    j = 2+jseg(seg):jseg(seg+1);
    ucv(1,j) = up(1,j);  vcv(1,j) = (vp(1,j) + vp(1,j-1))/2; 
    uav(1,j) = ucv(1,j); vav(1,j) = vcv(1,j);  
    j = jseg(seg+1)+1; 
    if j == J+1
      vcv(1,j) = vcv(1,j) + vp(1,j-1)/2; ucv(1,j) = ucv(1,j) + up(1,j)/2; 
    else
      ucv(1,j) = ucv(1,j) + up(1,j)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(1,j) = vcv(1,j) + vp(1,j-1)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1)); 
    end
    vav(1,j) = vav(1,j) + vp(1,j-1)/2; uav(1,j) = uav(1,j) + up(1,j)/2; 
  elseif xbc(1,seg) == 1	 	  	% No-slip at lower boundary
    % Do nothing
  else
    error('Wrong xbc')
  end
  if xbc(2,seg) == 2  			% Inflow at upper boundary
    j = 1+jseg(seg);
    if j == 1
      vcv(K+1,j) = vcv(K+1,j) + vupper(j)/2; 
      ucv(K+1,j) = ucv(K+1,j) + uupper(j)/2; 
    else
      ucv(K+1,j) = ucv(K+1,j) + uupper(j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(K+1,j) = vcv(K+1,j) + vupper(j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
    end
    vav(K+1,j) = vav(K+1,j) + vupper(j)/2; 
    uav(K+1,j) = uav(K+1,j) + uupper(j)/2; 
    j = 2+jseg(seg):jseg(seg+1);
    ucv(K+1,j) = uupper(j); vcv(K+1,j) = (vupper(j) + vupper(j-1))/2; 
    uav(K+1,j) = uupper(j); vav(K+1,j) = vcv(K+1,j); 
    j = jseg(seg+1)+1;
    if j == J+1
      vcv(K+1,j) = vcv(K+1,j) + vupper(j-1)/2; 
      ucv(K+1,j) = ucv(K+1,j) + uupper(j)/2;
    else 
      ucv(K+1,j) = ucv(K+1,j) + uupper(j)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(K+1,j) = vcv(K+1,j) + vupper(j-1)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1));
    end 
    vav(K+1,j) = vav(K+1,j) + vupper(j-1)/2; 
    uav(K+1,j) = uav(K+1,j) + uupper(j)/2;
  elseif xbc(2,seg) == 3		  	% Outflow at upper boundary
    j = 1+jseg(seg);
    if j == 1,  vcv(K+1,j) = vcv(K+1,j) + vp(K+1,j)/2; 
      ucv(K+1,j) = ucv(K+1,j) + up(K,j)/2; 
    else
      ucv(K+1,j) = ucv(K+1,j) + up(K,j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(K+1,j) = vcv(K+1,j) + vp(K+1,j)*DXV(1,j)/(DXV(1,j) + DXV(1,j-1)); 
    end
    vav(K+1,j) = vav(K+1,j) + vp(K+1,j)/2; 
    uav(K+1,j) = uav(K+1,j) + up(K,j)/2; 
    j = 2+jseg(seg):jseg(seg+1);
    ucv(K+1,j) = up(K,j); vcv(K+1,j) = (vp(K+1,j) + vp(K+1,j-1))/2; 
    uav(K+1,j) = up(K,j); vav(K+1,j) = vcv(K+1,j); 
    j = jseg(seg+1)+1;
    if j == J+1
      vcv(K+1,j) = vcv(K+1,j) + vp(K+1,j-1)/2; 
      ucv(K+1,j) = ucv(K+1,j) + up(K,j)/2;
    else 
      ucv(K+1,j) = ucv(K+1,j) + up(K,j)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1)); 
      vcv(K+1,j) = vcv(K+1,j) + vp(K+1,j-1)*DXV(1,j-1)/(DXV(1,j) + DXV(1,j-1));
    end 
    vav(K+1,j) = vav(K+1,j) + vp(K+1,j-1)/2; 
    uav(K+1,j) = uav(K+1,j) + up(K,j)/2;
  elseif xbc(2,seg) == 1		  	% No-slip at upper boundary
    % Do nothing
  else
    error('Wrong xbc')
  end
end

if central == 1				% Central scheme for inertia term
  
  ss = up.^2;  cu0(:,2:J) = ss(:,3:J+1) - ss(:,1:J-1);
  cu0(:,1) = ss(:,2) - ss(:,1);  cu0(:,J+1) = ss(:,J+1) - ss(:,J);
  uv = uav.*vcv;
  cu0 = cu0./(2*DXU) + (uv(2:K+1,:) - uv(1:K,:))./DYU;
  ss = vp.^2;  cv0(2:K,:) = ss(3:K+1,:) - ss(1:K-1,:);
  cv0(1,:) = ss(2,:) - ss(1,:); cv0(K+1,:) = ss(K+1,:) - ss(K,:);
  uv = ucv.*vav;
  cv0 = cv0./(2*DYV) + (uv(:,2:J+1) - uv(:,1:J))./DXV;

else					% Upwind scheme for inertia term
  
  uu = up; up = ((uu + abs(uu))/2).^2; um = ((uu - abs(uu))/2).^2; 
  vv = vp; vp = ((vv + abs(vv))/2).^2; vm = ((vv - abs(vv))/2).^2;   
  vpv = (vcv + abs(vcv))/2; vmv = (vcv - abs(vcv))/2;
  upv = (ucv + abs(ucv))/2; umv = (ucv - abs(ucv))/2;
  
  cu0(:,2:J) = up(:,2:J) + um(:,3:J+1) - up(:,1:J-1) - um(:,2:J);
  cu0(:,1) = um(:,2) + up(:,1) - uu(:,1).^2;  
  cu0(:,J+1) = uu(:,J+1).^2 - up(:,J) - um(:,J+1);
  
  uv = zeros(size(vpv)); 
  uv(2:K,:) = vpv(2:K,:).*uu(1:K-1,:) + vmv(2:K,:).*uu(2:K,:);
%  uv(1,:) = vpv(1,:).*ulower(:)' + vmv(1,:).*uu(1,:);
  uv(1,:) = vpv(1,:).*uav(1,:) + vmv(1,:).*uu(1,:);
%  uv(K+1,:) = vpv(K+1,:).*uu(K,:) + vmv(K+1,:).*uupper(:)';
  uv(K+1,:) = vpv(K+1,:).*uu(K,:) + vmv(K+1,:).*uav(K+1,:);
  cu0 = cu0./DXU + (uv(2:K+1,:) - uv(1:K,:))./DYU;
  
  cv0(2:K,:) = vp(2:K,:) + vm(3:K+1,:) - vp(1:K-1,:) - vm(2:K,:);
  cv0(1,:) = vp(1,:) + vm(2,:) - vv(1,:).^2;
  cv0(K+1,:) = vv(K+1,:).^2 - vm(K+1,:) - vp(K,:); 
  uv(:,2:J) = upv(:,2:J).*vv(:,1:J-1) + umv(:,2:J).*vv(:,2:J);
%  uv(:,1) = upv(:,1).*vleft(:) + umv(:,1).*vv(:,1);
  uv(:,1) = upv(:,1).*vav(:,1) + umv(:,1).*vv(:,1);
%  uv(:,J+1) = upv(:,J+1).*vv(:,J) + umv(:,J+1).*vright(:);
  uv(:,J+1) = upv(:,J+1).*vv(:,J) + umv(:,J+1).*vav(:,J+1);
  cv0 = cv0./DYV + (uv(:,2:J+1) - uv(:,1:J))./DXV;
end

for seg = 1:length(ybc(1,:))
  if (ybc(1,seg) == 1)|(ybc(1,seg) == 2)   % No-slip or inflow at left boundary
    k = 1+kseg(seg):kseg(seg+1);  cu0(k,1) = 0;
  end
  if (ybc(2,seg) == 1)|(ybc(2,seg) == 2)   % No-slip or inflow at right boundary
    k = 1+kseg(seg):kseg(seg+1);  cu0(k,J+1) = 0;
  end
end
for seg = 1:length(xbc(1,:))
  if (xbc(1,seg) == 1)|(xbc(1,seg) == 2)  % No-slip or inflow at lower boundary
    j = 1+jseg(seg):jseg(seg+1); cv0(1,j) = 0;
  end
  if (xbc(2,seg) == 1)|(xbc(2,seg) == 2)  % No-slip or inflow at upper boundary
    j = 1+jseg(seg):jseg(seg+1); cv0(K+1,j) = 0;
  end
end

cu0 = cu0'; cu0 = cu0(:); cv0 = cv0'; cv0 = cv0(:);
if n<2, tijd = toc; disp(['inertia_term time = ',num2str(tijd)]), end
