%PROBLEM SPECIFICATION
% Specification of parameters, grid and boundary conditions

global Re  xseglen  yseglen
tpic = zeros(1,10); kpic = 0;	% Used only for geval == 3 for extra plot
%tic
if geval == 1	% Horizontal Poiseuille flow to the right
  dt = 2;	% Time step
  tend = 30;	% End time
  Re = 200;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1/2;  % Omega-scheme for time-stepping
  umax = 1.5;	% Estimates of twice the max. velocity components required for 
  vmax = 0.1;	% stability condition for Adams-Bashforth-crank-Nicolson scheme
  		% (not used in program ns1)
  
  xseglen = [1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1];	% Similar to xseglen in y-direction.
  nx = [2];	% Number of cells along x-segments.
  ny = [2];	% Number of cells along y-segments.
		
  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  % (second row) horizontal segments in order of increasing x:
   
  xbc = [1; 1];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [2; 3];
    
elseif geval == 2	% Vertical Poiseuille flow upward
  dt = 0.5;	% Time step
  tend = 45;	% End time
  Re = 200;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1/2;  % Omega-scheme for time-stepping
  umax = 0;	% Estimates of twice the max. velocity components required for 
  vmax = 1.5;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

  xseglen = [1,1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1,1];	% Similar to xseglen in y-direction.
  nx = [8,8];	% Number of cells along x-segments.
  ny = [8,8];	% Number of cells along y-segments.
		
  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [2,2; 3,3];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [1,1; 1,1];
  
elseif geval == 3	% Backward facing step, horizontal; see also geval == 10
  dt = 0.30;	% Time step
  tend = 150;	% End time
  Re = 400;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1;  % Omega-scheme for time-stepping
  umax = 3;	% Estimates of twice the max. velocity components required for 
  vmax = 0.3;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

  xseglen = [25];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1,1];	% Similar to xseglen in y-direction.
  nx = [70];	% Number of cells along x-segments.
  ny = [30,30];	% Number of cells along y-segments.

  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [1;1];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [1,2; 3,3];
  tpic(1) = 0.4*tend; tpic(2) = 0.6*tend; tpic(3) = 0.7*tend;
  tpic(4) = 0.8*tend; tpic(5) = 0.9*tend; tpic(6) = 0.92*tend;
  tpic(7) = 0.94*tend; tpic(8) = 0.96*tend; tpic(9) = 0.98*tend;
  tpic(10) = 2*tend;
  
elseif geval == 4	% Driven cavity
  dt = 0.08;	% Time step
  tend = 140;	% End time
  Re = 6000;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1;  % Omega-scheme for time-stepping
  umax = 2;	% Estimates of twice the max. velocity components, required for 
  vmax = 0;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

%  xseglen = [1];% Enter segment lengths of horizontal boundary in
  xseglen = [0.2,0.6,0.2];% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
%  yseglen = [1];	% Similar to xseglen in y-direction.
  yseglen = [0.2,0.6,0.2];	% Similar to xseglen in y-direction.
%  nx = [84];	% Number of cells along x-segments.
%  ny = [84];	% Number of cells along y-segments.
  nx = [21,22,21];	% Number of cells along x-segments.
  ny = [21,22,21];	% Number of cells along y-segments.
%  nx = [4,4,4];	% Number of cells along x-segments.
%  ny = [4,4,4];	% Number of cells along y-segments.
  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
%  xbc = [1; 2];
  xbc = [1,1,1; 2,2,2];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
%  ybc = [1; 1];
  ybc = [1,1,1; 1,1,1];
  
elseif geval == 5	% Uniform flow under angle alpha in [0,pi/2] without
			% segmentation
  alpha = pi/3;
  dt = 0.04;	% Time step
  tend = 11;	% End time
  Re = 1;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1/2;  % Omega-scheme for time-stepping
  umax = 2*abs(cos(alpha));% Estimates of twice the max. velocity components, 
  			   % required for stability condition
  vmax = 2*abs(sin(alpha));% for Adams-Bashforth-Crank-Nicolson scheme
  			   % (not used in program ns1)

  xseglen = [1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1];	% Similar to xseglen in y-direction.
  nx = [2];	% Number of cells along x-segments.
  ny = [2];	% Number of cells along y-segments.

  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [2; 3];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [2; 3];
  
elseif geval == 6	% Uniform flow under angle alpha in [0,pi/2] with
			% segmentation
  alpha = pi/6;
  dt = 3;	% Time step
  tend = 60;	% End time
  Re = 100000;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1/2;  % Omega-scheme for time-stepping
  umax = 2*abs(cos(alpha));% Estimates of twice the max. velocity components,
  			   % required for stability condition
  vmax = 2*abs(sin(alpha));% for Adams-Bashforth-Crank-Nicolson scheme
  			   % (not used in program ns1)

  xseglen = [1,1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1,1];	% Similar to xseglen in y-direction.
  nx = [1,1];	% Number of cells along x-segments.
  ny = [1,1];	% Number of cells along y-segments.

  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [2,2; 3,3];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [2,2; 3,3];
  
elseif geval == 7	% Horizontal Poiseuille flow to the left
  dt = 0.5;	% Time step
  tend = 160;	% End time
  Re = 10000;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1;    % Omega-scheme for time-stepping
  umax = 1.5;	% Estimates of twice the max. velocity components, required for
  vmax = 0;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

  xseglen = [1,1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1,1];	% Similar to xseglen in y-direction.
  nx = [10,10];	% Number of cells along x-segments.
  ny = [10,10];	% Number of cells along y-segments.
		
  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [1,1; 1,1];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [3,3; 2,2];    
  
elseif geval == 8	% Uniform horizontal flow with inflow bound. cond. on
			% three sides 
  dt = 0.5;	% Time step
  tend = 16;	% End time
  Re = 1;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1;    % Omega-scheme for time-stepping
  umax = 1.0;	% Estimates of twice the max. velocity components, required for
  vmax = 0;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

  xseglen = [1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1];	% Similar to xseglen in y-direction.
  nx = [3];	% Number of cells along x-segments.
  ny = [3];	% Number of cells along y-segments.
		
  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [2; 2];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [2; 3];    
elseif geval == 9	% Uniform horizontal flow with outflow bound. cond. on
			% three sides 
  dt = 0.5;	% Time step
  tend = 16;	% End time
  Re = 1;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1;    % Omega-scheme for time-stepping
  umax = 1.0;	% Estimates of twice the max. velocity components, required for
  vmax = 0;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

  xseglen = [1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [1];	% Similar to xseglen in y-direction.
  nx = [3];	% Number of cells along x-segments.
  ny = [3];	% Number of cells along y-segments.
		
  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [3; 3];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [2; 3];    
elseif geval == 10	% Backward facing step, vertial; see also geval == 10
  dt = 0.60;	% Time step
  tend = 60;	% End time
  Re = 400;	% Reynolds number based on unit length and unit velocity
  central = 1;	% Enter 1 for central scheme or something else for upwind scheme
  omega = 1;  % Omega-scheme for time-stepping
  umax = 3;	% Estimates of twice the max. velocity components required for 
  vmax = 0.3;	% stability condition for Adams-Bashforth-Crank-Nicolson scheme
  		% (not used in program ns1)

  xseglen = [1,1];	% Enter segment lengths of horizontal boundary in
		% order of increasing x. Number of segments is arbitrary.
		% These segments are used for grid generation and boundary 
		% conditions. 
  yseglen = [25];	% Similar to xseglen in y-direction.
  nx = [10,10];	% Number of cells along x-segments.
  ny = [50];	% Number of cells along y-segments.

  % Type of boundary condition: 1: no-slip
  %				2: inflow
  %				3: outflow		
  % Give type of boundary condition along lower (first row) and upper 
  %(second row) horizontal segments in order of increasing x:
   
  xbc = [2,1;3,3];
   	
  % Give type of boundary condition along left (first row) and right 
  % (second row) vertical segments in order of increasing y:
  
  ybc = [1; 1];
  tpic(1) = 0.4*tend; tpic(2) = 0.6*tend; tpic(3) = 0.7*tend;
  tpic(4) = 0.8*tend; tpic(5) = 0.9*tend; tpic(6) = 0.92*tend;
  tpic(7) = 0.94*tend; tpic(8) = 0.96*tend; tpic(9) = 0.98*tend;
  tpic(10) = 2*tend;
  
else
  error('Wrong value in input for parameter geval')  	
end
		
if (size(xseglen)~=size(nx))|(size(yseglen)~=size(ny))
  error('Wrong correspondence between xseglen,nx or yseglen,ny in problem_specification')
end
if (size(xbc(1,:))~=size(nx))|(size(ybc(1,:))~=size(nx))
  error('Wrong correspondence between nx,xbc or ny,ybc in problem_specification')
end

%tijd = toc; disp(['problem_specification time = ',num2str(tijd)])
